package java.awt;

import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;
import java.beans.ConstructorProperties;
import java.awt.color.ColorSpace;
import java.io.Serializable;

public class Color implements Paint, Serializable
{
    public static final Color white;
    public static final Color WHITE;
    public static final Color lightGray;
    public static final Color LIGHT_GRAY;
    public static final Color gray;
    public static final Color GRAY;
    public static final Color darkGray;
    public static final Color DARK_GRAY;
    public static final Color black;
    public static final Color BLACK;
    public static final Color red;
    public static final Color RED;
    public static final Color pink;
    public static final Color PINK;
    public static final Color orange;
    public static final Color ORANGE;
    public static final Color yellow;
    public static final Color YELLOW;
    public static final Color green;
    public static final Color GREEN;
    public static final Color magenta;
    public static final Color MAGENTA;
    public static final Color cyan;
    public static final Color CYAN;
    public static final Color blue;
    public static final Color BLUE;
    int value;
    private float[] frgbvalue;
    private float[] fvalue;
    private float falpha;
    private ColorSpace cs;
    private static final long serialVersionUID = 118526816881161077L;
    private static final double FACTOR = 0.7;
    
    private static native void initIDs();
    
    private static void testColorValueRange(int n, int n2, int n3, int n4) {
        int n5 = 0;
        String str = "";
        if (n4 < 0 || n4 > 255) {
            n5 = 1;
            str += " Alpha";
        }
        if (n < 0 || n > 255) {
            n5 = 1;
            str += " Red";
        }
        if (n2 < 0 || n2 > 255) {
            n5 = 1;
            str += " Green";
        }
        if (n3 < 0 || n3 > 255) {
            n5 = 1;
            str += " Blue";
        }
        if (n5 == 1) {
            throw new IllegalArgumentException("Color parameter outside of expected range:" + str);
        }
        return;
    }
    
    private static void testColorValueRange(float n, float n2, float n3, float n4) {
        int n5 = 0;
        String str = "";
        if ((double)n4 < 0.0 || (double)n4 > 1.0) {
            n5 = 1;
            str += " Alpha";
        }
        if ((double)n < 0.0 || (double)n > 1.0) {
            n5 = 1;
            str += " Red";
        }
        if ((double)n2 < 0.0 || (double)n2 > 1.0) {
            n5 = 1;
            str += " Green";
        }
        if ((double)n3 < 0.0 || (double)n3 > 1.0) {
            n5 = 1;
            str += " Blue";
        }
        if (n5 == 1) {
            throw new IllegalArgumentException("Color parameter outside of expected range:" + str);
        }
        return;
    }
    
    public Color(int n, int n2, int n3) {
        this(n, n2, n3, 255);
    }
    
    @ConstructorProperties({ "red", "green", "blue", "alpha" })
    public Color(int n, int n2, int n3, int n4) {
        this.frgbvalue = null;
        this.fvalue = null;
        this.falpha = 0.0f;
        this.cs = null;
        this.value = ((n4 & 0xFF) << 24 | (n & 0xFF) << 16 | (n2 & 0xFF) << 8 | (n3 & 0xFF) << 0);
        testColorValueRange(n, n2, n3, n4);
    }
    
    public Color(int n) {
        this.frgbvalue = null;
        this.fvalue = null;
        this.falpha = 0.0f;
        this.cs = null;
        this.value = (0xFF000000 | n);
    }
    
    public Color(int value, boolean b) {
        this.frgbvalue = null;
        this.fvalue = null;
        this.falpha = 0.0f;
        this.cs = null;
        if (b) {
            this.value = value;
        }
        else {
            this.value = (0xFF000000 | value);
        }
    }
    
    public Color(float n, float n2, float n3) {
        this((int)((double)(n * 255.0f) + 0.5), (int)((double)(n2 * 255.0f) + 0.5), (int)((double)(n3 * 255.0f) + 0.5));
        testColorValueRange(n, n2, n3, 1.0f);
        (this.frgbvalue = new float[3])[0] = n;
        this.frgbvalue[1] = n2;
        this.frgbvalue[2] = n3;
        this.falpha = 1.0f;
        this.fvalue = this.frgbvalue;
    }
    
    public Color(float n, float n2, float n3, float falpha) {
        this((int)((double)(n * 255.0f) + 0.5), (int)((double)(n2 * 255.0f) + 0.5), (int)((double)(n3 * 255.0f) + 0.5), (int)((double)(falpha * 255.0f) + 0.5));
        (this.frgbvalue = new float[3])[0] = n;
        this.frgbvalue[1] = n2;
        this.frgbvalue[2] = n3;
        this.falpha = falpha;
        this.fvalue = this.frgbvalue;
    }
    
    public Color(ColorSpace cs, float[] array, float falpha) {
        this.frgbvalue = null;
        this.fvalue = null;
        this.falpha = 0.0f;
        this.cs = null;
        boolean n = false;
        String str = "";
        int numComponents = cs.getNumComponents();
        this.fvalue = new float[numComponents];
        for (int i = 0; i < numComponents; ++i) {
            if ((double)array[i] < 0.0 || (double)array[i] > 1.0) {
                n = true;
                str = str + "Component " + i + " ";
            }
            else {
                this.fvalue[i] = array[i];
            }
        }
        if ((double)falpha < 0.0 || (double)falpha > 1.0) {
            n = true;
            str += "Alpha";
        }
        else {
            this.falpha = falpha;
        }
        if (!(n == 0)) {
            throw new IllegalArgumentException("Color parameter outside of expected range: " + str);
        }
        this.frgbvalue = cs.toRGB(this.fvalue);
        this.cs = cs;
        this.value = (((int)(this.falpha * 255.0f) & 0xFF) << 24 | ((int)(this.frgbvalue[0] * 255.0f) & 0xFF) << 16 | ((int)(this.frgbvalue[1] * 255.0f) & 0xFF) << 8 | ((int)(this.frgbvalue[2] * 255.0f) & 0xFF) << 0);
        return;
    }
    
    public int getRed() {
        return this.getRGB() >> 16 & 0xFF;
    }
    
    public int getGreen() {
        return this.getRGB() >> 8 & 0xFF;
    }
    
    public int getBlue() {
        return this.getRGB() >> 0 & 0xFF;
    }
    
    public int getAlpha() {
        return this.getRGB() >> 24 & 0xFF;
    }
    
    public int getRGB() {
        return this.value;
    }
    
    public Color brighter() {
        int red = this.getRed();
        int green = this.getGreen();
        int blue = this.getBlue();
        int alpha = this.getAlpha();
        int n = 3;
        if (red == 0 && green == 0 && blue == 0) {
            return new Color(n, n, n, alpha);
        }
        if (red > 0 && red < n) {
            red = n;
        }
        if (green > 0 && green < n) {
            green = n;
        }
        if (blue > 0 && blue < n) {
            blue = n;
        }
        return new Color(Math.min((int)((double)red / 0.7), 255), Math.min((int)((double)green / 0.7), 255), Math.min((int)((double)blue / 0.7), 255), alpha);
    }
    
    public Color darker() {
        return new Color(Math.max((int)((double)this.getRed() * 0.7), 0), Math.max((int)((double)this.getGreen() * 0.7), 0), Math.max((int)((double)this.getBlue() * 0.7), 0), this.getAlpha());
    }
    
    @Override
    public int hashCode() {
        return this.value;
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof Color && ((Color)o).getRGB() == this.getRGB();
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[r=" + this.getRed() + ",g=" + this.getGreen() + ",b=" + this.getBlue() + "]";
    }
    
    public static Color decode(String nm) throws NumberFormatException {
        int intValue = (int)Integer.decode(nm);
        return new Color(intValue >> 16 & 0xFF, intValue >> 8 & 0xFF, intValue & 0xFF);
    }
    
    public static Color getColor(String s) {
        return getColor(s, null);
    }
    
    public static Color getColor(String nm, Color color) {
        Integer integer = Integer.getInteger(nm);
        if (integer == null) {
            return color;
        }
        int intValue = (int)integer;
        return new Color(intValue >> 16 & 0xFF, intValue >> 8 & 0xFF, intValue & 0xFF);
    }
    
    public static Color getColor(String nm, int n) {
        Integer integer = Integer.getInteger(nm);
        n = ((integer != null) ? ((int)integer) : n);
        return new Color(n >> 16 & 0xFF, n >> 8 & 0xFF, n >> 0 & 0xFF);
    }
    
    public static int HSBtoRGB(float n, float n2, float n3) {
        int n4 = 0;
        int n5 = 0;
        int n6 = 0;
        if (n2 == 0.0f) {
            n5 = (n4 = (n6 = (int)(n3 * 255.0f + 0.5f)));
        }
        else {
            float n7 = (n - (float)Math.floor((double)n)) * 6.0f;
            float n8 = n7 - (float)Math.floor((double)n7);
            float n9 = n3 * (1.0f - n2);
            float n10 = n3 * (1.0f - n2 * n8);
            float n11 = n3 * (1.0f - n2 * (1.0f - n8));
            switch ((int)n7) {
                case 0: {
                    n4 = (int)(n3 * 255.0f + 0.5f);
                    n5 = (int)(n11 * 255.0f + 0.5f);
                    n6 = (int)(n9 * 255.0f + 0.5f);
                    break;
                }
                case 1: {
                    n4 = (int)(n10 * 255.0f + 0.5f);
                    n5 = (int)(n3 * 255.0f + 0.5f);
                    n6 = (int)(n9 * 255.0f + 0.5f);
                    break;
                }
                case 2: {
                    n4 = (int)(n9 * 255.0f + 0.5f);
                    n5 = (int)(n3 * 255.0f + 0.5f);
                    n6 = (int)(n11 * 255.0f + 0.5f);
                    break;
                }
                case 3: {
                    n4 = (int)(n9 * 255.0f + 0.5f);
                    n5 = (int)(n10 * 255.0f + 0.5f);
                    n6 = (int)(n3 * 255.0f + 0.5f);
                    break;
                }
                case 4: {
                    n4 = (int)(n11 * 255.0f + 0.5f);
                    n5 = (int)(n9 * 255.0f + 0.5f);
                    n6 = (int)(n3 * 255.0f + 0.5f);
                    break;
                }
                case 5: {
                    n4 = (int)(n3 * 255.0f + 0.5f);
                    n5 = (int)(n9 * 255.0f + 0.5f);
                    n6 = (int)(n10 * 255.0f + 0.5f);
                    break;
                }
            }
        }
        return 0xFF000000 | n4 << 16 | n5 << 8 | n6 << 0;
    }
    
    public static float[] RGBtoHSB(int n, int n2, int n3, float[] array) {
        if (array == null) {
            array = new float[3];
        }
        n2 = ((n > n2) ? n : n2);
        if (n3 > n2) {
            n2 = n3;
        }
        n2 = ((n < n2) ? n : n2);
        if (n3 < n2) {
            n2 = n3;
        }
        float n4 = (float)n2 / 255.0f;
        float n5;
        if (n2 != 0) {
            n5 = (float)(n2 - n2) / (float)n2;
        }
        else {
            n5 = 0.0f;
        }
        float n10;
        if (n5 == 0.0f) {
            n10 = 0.0f;
        }
        else {
            float n6 = (float)(n2 - n) / (float)(n2 - n2);
            float n7 = (float)(n2 - n2) / (float)(n2 - n2);
            float n8 = (float)(n2 - n3) / (float)(n2 - n2);
            float n9;
            if (n == n2) {
                n9 = n8 - n7;
            }
            else if (n2 == n2) {
                n9 = 2.0f + n6 - n8;
            }
            else {
                n9 = 4.0f + n7 - n6;
            }
            n10 = n9 / 6.0f;
            if (n10 < 0.0f) {
                ++n10;
            }
        }
        array[0] = n10;
        array[1] = n5;
        array[2] = n4;
        return array;
    }
    
    public static Color getHSBColor(float n, float n2, float n3) {
        return new Color(HSBtoRGB(n, n2, n3));
    }
    
    public float[] getRGBComponents(float[] array) {
        float[] array2;
        if (array == null) {
            array2 = new float[4];
        }
        else {
            array2 = array;
        }
        if (this.frgbvalue == null) {
            array2[0] = (float)this.getRed() / 255.0f;
            array2[1] = (float)this.getGreen() / 255.0f;
            array2[2] = (float)this.getBlue() / 255.0f;
            array2[3] = (float)this.getAlpha() / 255.0f;
            return array2;
        }
        array2[0] = this.frgbvalue[0];
        array2[1] = this.frgbvalue[1];
        array2[2] = this.frgbvalue[2];
        array2[3] = this.falpha;
        return array2;
    }
    
    public float[] getRGBColorComponents(float[] array) {
        float[] array2;
        if (array == null) {
            array2 = new float[3];
        }
        else {
            array2 = array;
        }
        if (this.frgbvalue == null) {
            array2[0] = (float)this.getRed() / 255.0f;
            array2[1] = (float)this.getGreen() / 255.0f;
            array2[2] = (float)this.getBlue() / 255.0f;
            return array2;
        }
        array2[0] = this.frgbvalue[0];
        array2[1] = this.frgbvalue[1];
        array2[2] = this.frgbvalue[2];
        return array2;
    }
    
    public float[] getComponents(float[] array) {
        if (this.fvalue == null) {
            return this.getRGBComponents(array);
        }
        int length = this.fvalue.length;
        float[] array2;
        if (array == null) {
            array2 = new float[length + 1];
        }
        else {
            array2 = array;
        }
        for (int i = 0; i < length; ++i) {
            array2[i] = this.fvalue[i];
        }
        array2[length] = this.falpha;
        return array2;
    }
    
    public float[] getColorComponents(float[] array) {
        if (this.fvalue == null) {
            return this.getRGBColorComponents(array);
        }
        int length = this.fvalue.length;
        float[] array2;
        if (array == null) {
            array2 = new float[length];
        }
        else {
            array2 = array;
        }
        for (int i = 0; i < length; ++i) {
            array2[i] = this.fvalue[i];
        }
        return array2;
    }
    
    public float[] getComponents(ColorSpace colorSpace, float[] array) {
        if (this.cs == null) {
            this.cs = ColorSpace.getInstance(1000);
        }
        float[] fvalue;
        if (this.fvalue == null) {
            fvalue = new float[] { (float)this.getRed() / 255.0f, (float)this.getGreen() / 255.0f, (float)this.getBlue() / 255.0f };
        }
        else {
            fvalue = this.fvalue;
        }
        float[] fromCIEXYZ = colorSpace.fromCIEXYZ(this.cs.toCIEXYZ(fvalue));
        if (array == null) {
            array = new float[fromCIEXYZ.length + 1];
        }
        for (int i = 0; i < fromCIEXYZ.length; ++i) {
            array[i] = fromCIEXYZ[i];
        }
        if (this.fvalue == null) {
            array[fromCIEXYZ.length] = (float)this.getAlpha() / 255.0f;
            return array;
        }
        array[fromCIEXYZ.length] = this.falpha;
        return array;
    }
    
    public float[] getColorComponents(ColorSpace colorSpace, float[] array) {
        if (this.cs == null) {
            this.cs = ColorSpace.getInstance(1000);
        }
        float[] fvalue;
        if (this.fvalue == null) {
            fvalue = new float[] { (float)this.getRed() / 255.0f, (float)this.getGreen() / 255.0f, (float)this.getBlue() / 255.0f };
        }
        else {
            fvalue = this.fvalue;
        }
        float[] fromCIEXYZ = colorSpace.fromCIEXYZ(this.cs.toCIEXYZ(fvalue));
        if (array == null) {
            return fromCIEXYZ;
        }
        for (int i = 0; i < fromCIEXYZ.length; ++i) {
            array[i] = fromCIEXYZ[i];
        }
        return array;
    }
    
    public ColorSpace getColorSpace() {
        if (this.cs == null) {
            this.cs = ColorSpace.getInstance(1000);
        }
        return this.cs;
    }
    
    @Override
    public synchronized PaintContext createContext(ColorModel colorModel, Rectangle rectangle, Rectangle2D rectangle2D, AffineTransform affineTransform, RenderingHints renderingHints) {
        return new ColorPaintContext(this.getRGB(), colorModel);
    }
    
    @Override
    public int getTransparency() {
        int alpha = this.getAlpha();
        if (alpha == 255) {
            return 1;
        }
        if (alpha == 0) {
            return 2;
        }
        return 3;
    }
    
    static {
        white = new Color(255, 255, 255);
        WHITE = Color.white;
        lightGray = new Color(192, 192, 192);
        LIGHT_GRAY = Color.lightGray;
        gray = new Color(128, 128, 128);
        GRAY = Color.gray;
        darkGray = new Color(64, 64, 64);
        DARK_GRAY = Color.darkGray;
        black = new Color(0, 0, 0);
        BLACK = Color.black;
        red = new Color(255, 0, 0);
        RED = Color.red;
        pink = new Color(255, 175, 175);
        PINK = Color.pink;
        orange = new Color(255, 200, 0);
        ORANGE = Color.orange;
        yellow = new Color(255, 255, 0);
        YELLOW = Color.yellow;
        green = new Color(0, 255, 0);
        GREEN = Color.green;
        magenta = new Color(255, 0, 255);
        MAGENTA = Color.magenta;
        cyan = new Color(0, 255, 255);
        CYAN = Color.cyan;
        blue = new Color(0, 0, 255);
        BLUE = Color.blue;
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
    }
}
